#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	mat3 TBN;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D normalTexture;
uniform bool normal_mapping;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	

	//cel shading
	float levels = 4.0;
	float brightness = max(0, dot(n, l));
	float level = floor(brightness * levels);
	brightness = level/levels;

	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);

	//cel shading part 2
	float dampedFactor = pow(max(0, dot(r, v)), alpha);
	level = floor(dampedFactor * levels);
	dampedFactor = level / levels;

	return (diffuseF * diffuseC * brightness + specularF * specularC * dampedFactor) * att; 
}

void main() {
	vec3 n;
	if (normal_mapping){
		n = texture(normalTexture, vert.uv).rgb;
		n = normalize(n * 2.0 - 1.0);   
		n = normalize(vert.TBN * n);
	}
	else{
		n = normalize(vert.TBN[2]);
	}
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	color = vec4(texColor * materialCoefficients.x, 1); // ambient

	//cel shading part 3
	float levels = 4.0;
	float amount = (color.r + color.g + color.b) / 3.0;
	amount = floor(amount * levels) / levels;
	color.r = amount * color.r + 0.1;
	color.g = amount * color.g + 0.1;
	color.b = amount * color.b + 0.1;
	
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
	
}

